# -----------------------------------------------------------------------------
# Copyright © 2019- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtChart classes and functions."""

from . import (
    PYQT5,
    PYQT6,
    PYSIDE2,
    PYSIDE6,
    QtBindingsNotFoundError,
    QtModuleNotInstalledError,
)

if PYQT5:
    try:
        from PyQt5.QtChart import *
        from PyQt5 import QtChart as QtCharts
    except ModuleNotFoundError as error:
        raise QtModuleNotInstalledError(
            name='QtCharts', missing_package='PyQtChart'
        ) from error
elif PYQT6:
    try:
        from PyQt6.QtCharts import *
        from PyQt6 import QtCharts
    except ModuleNotFoundError as error:
        raise QtModuleNotInstalledError(
            name='QtCharts', missing_package='PyQt6-Charts'
        ) from error
elif PYSIDE2:
    from PySide2.QtCharts import *

    # https://bugreports.qt.io/projects/PYSIDE/issues/PYSIDE-1026
    import PySide2.QtCharts as __temp
    import inspect

    for __name in inspect.getmembers(__temp.QtCharts):
        globals()[__name[0]] = __name[1]
elif PYSIDE6:
    from PySide6.QtCharts import *
    from PySide6 import QtCharts
else:
    raise QtBindingsNotFoundError()



import pytest


def test_qt3dlogic():
    """Test the qtpy.Qt3DLogic namespace"""
    Qt3DLogic = pytest.importorskip("qtpy.Qt3DLogic")

    assert Qt3DLogic.QLogicAspect is not None
    assert Qt3DLogic.QFrameAction is not None



import pytest
from qtpy import PYSIDE6, PYQT6


def test_qtsvg():
    """Test the qtpy.QtSvg namespace"""
    from qtpy import QtSvg

    if not (PYSIDE6 or PYQT6):
        assert QtSvg.QGraphicsSvgItem is not None
        assert QtSvg.QSvgWidget is not None
    assert QtSvg.QSvgGenerator is not None
    assert QtSvg.QSvgRenderer is not None



import pytest


def test_qtquickcontrols2():
    """Test the qtpy.QtQuickControls2 namespace"""
    QtQuickControls2 = pytest.importorskip("qtpy.QtQuickControls2")

    assert QtQuickControls2.QQuickStyle is not None



"""Test the QtPy CLI."""

import subprocess
import sys

import pytest

import qtpy


SUBCOMMANDS = [
    [],
    ['mypy-args'],
]


@pytest.mark.parametrize(
    argnames=['subcommand'],
    argvalues=[[subcommand] for subcommand in SUBCOMMANDS],
    ids=[' '.join(subcommand) for subcommand in SUBCOMMANDS],
)
def test_cli_help_does_not_fail(subcommand):
    subprocess.run(
        [sys.executable, '-m', 'qtpy', *subcommand, '--help'], check=True,
    )


def test_cli_version():
    output = subprocess.run(
        [sys.executable, '-m', 'qtpy', '--version'],
        capture_output=True,
        check=True,
        encoding='utf-8',
    )
    assert output.stdout.strip().split()[-1] == qtpy.__version__


def test_cli_mypy_args():
    output = subprocess.run(
        [sys.executable, '-m', 'qtpy', 'mypy-args'],
        capture_output=True,
        check=True,
        encoding='utf-8',
    )

    if qtpy.PYQT5:
        expected = ' '.join([
            '--always-true=PYQT5',
            '--always-false=PYSIDE2',
            '--always-false=PYQT6',
            '--always-false=PYSIDE6',
        ])
    elif qtpy.PYSIDE2:
        expected = ' '.join([
            '--always-false=PYQT5',
            '--always-true=PYSIDE2',
            '--always-false=PYQT6',
            '--always-false=PYSIDE6',
        ])
    elif qtpy.PYQT6:
        expected = ' '.join([
            '--always-false=PYQT5',
            '--always-false=PYSIDE2',
            '--always-true=PYQT6',
            '--always-false=PYSIDE6',
        ])
    elif qtpy.PYSIDE6:
        expected = ' '.join([
            '--always-false=PYQT5',
            '--always-false=PYSIDE2',
            '--always-false=PYQT6',
            '--always-true=PYSIDE6',
        ])
    else:
        assert False, 'No valid API to test'

    assert output.stdout.strip() == expected.strip()



import pytest
from qtpy import PYQT5, PYSIDE2


@pytest.mark.skipif(not (PYQT5 or PYSIDE2), reason="Only available in Qt5 bindings")
def test_qtwebsockets():
    """Test the qtpy.QtWebSockets namespace"""
    from qtpy import QtWebSockets

    assert QtWebSockets.QMaskGenerator is not None
    assert QtWebSockets.QWebSocket is not None
    assert QtWebSockets.QWebSocketCorsAuthenticator is not None
    assert QtWebSockets.QWebSocketProtocol is not None
    assert QtWebSockets.QWebSocketServer is not None



import pytest


def test_qtdatavisualization():
    """Test the qtpy.QtDataVisualization namespace"""
    # Using import skip here since with Python 3 you need to install another package
    # besides the base `PyQt5` or `PySide2`.
    # For example in the case of `PyQt5` you need `PyQtDataVisualization`

    # QtDataVisualization
    QtDataVisualization = pytest.importorskip("qtpy.QtDataVisualization")
    assert QtDataVisualization.QScatter3DSeries is not None
    assert QtDataVisualization.QSurfaceDataItem is not None
    assert QtDataVisualization.QSurface3DSeries is not None
    assert QtDataVisualization.QAbstract3DInputHandler is not None
    assert QtDataVisualization.QHeightMapSurfaceDataProxy is not None
    assert QtDataVisualization.QAbstractDataProxy is not None
    assert QtDataVisualization.Q3DCamera is not None
    assert QtDataVisualization.QAbstract3DGraph is not None
    assert QtDataVisualization.QCustom3DVolume is not None
    assert QtDataVisualization.Q3DInputHandler is not None
    assert QtDataVisualization.QBarDataProxy is not None
    assert QtDataVisualization.QSurfaceDataProxy is not None
    assert QtDataVisualization.QScatterDataItem is not None
    assert QtDataVisualization.Q3DLight is not None
    assert QtDataVisualization.QScatterDataProxy is not None
    assert QtDataVisualization.QValue3DAxis is not None
    assert QtDataVisualization.Q3DBars is not None
    assert QtDataVisualization.QBarDataItem is not None
    assert QtDataVisualization.QItemModelBarDataProxy is not None
    assert QtDataVisualization.Q3DTheme is not None
    assert QtDataVisualization.QCustom3DItem is not None
    assert QtDataVisualization.QItemModelScatterDataProxy is not None
    assert QtDataVisualization.QValue3DAxisFormatter is not None
    assert QtDataVisualization.QItemModelSurfaceDataProxy is not None
    assert QtDataVisualization.Q3DScatter is not None
    assert QtDataVisualization.QTouch3DInputHandler is not None
    assert QtDataVisualization.QBar3DSeries is not None
    assert QtDataVisualization.QAbstract3DAxis is not None
    assert QtDataVisualization.Q3DScene is not None
    assert QtDataVisualization.QCategory3DAxis is not None
    assert QtDataVisualization.QAbstract3DSeries is not None
    assert QtDataVisualization.Q3DObject is not None
    assert QtDataVisualization.QCustom3DLabel is not None
    assert QtDataVisualization.Q3DSurface is not None
    assert QtDataVisualization.QLogValue3DAxisFormatter is not None

    # QtDatavisualization
    QtDatavisualization = pytest.importorskip("qtpy.QtDatavisualization")

    assert QtDatavisualization.QScatter3DSeries is not None
    assert QtDatavisualization.QSurfaceDataItem is not None
    assert QtDatavisualization.QSurface3DSeries is not None
    assert QtDatavisualization.QAbstract3DInputHandler is not None
    assert QtDatavisualization.QHeightMapSurfaceDataProxy is not None
    assert QtDatavisualization.QAbstractDataProxy is not None
    assert QtDatavisualization.Q3DCamera is not None
    assert QtDatavisualization.QAbstract3DGraph is not None
    assert QtDatavisualization.QCustom3DVolume is not None
    assert QtDatavisualization.Q3DInputHandler is not None
    assert QtDatavisualization.QBarDataProxy is not None
    assert QtDatavisualization.QSurfaceDataProxy is not None
    assert QtDatavisualization.QScatterDataItem is not None
    assert QtDatavisualization.Q3DLight is not None
    assert QtDatavisualization.QScatterDataProxy is not None
    assert QtDatavisualization.QValue3DAxis is not None
    assert QtDatavisualization.Q3DBars is not None
    assert QtDatavisualization.QBarDataItem is not None
    assert QtDatavisualization.QItemModelBarDataProxy is not None
    assert QtDatavisualization.Q3DTheme is not None
    assert QtDatavisualization.QCustom3DItem is not None
    assert QtDatavisualization.QItemModelScatterDataProxy is not None
    assert QtDatavisualization.QValue3DAxisFormatter is not None
    assert QtDatavisualization.QItemModelSurfaceDataProxy is not None
    assert QtDatavisualization.Q3DScatter is not None
    assert QtDatavisualization.QTouch3DInputHandler is not None
    assert QtDatavisualization.QBar3DSeries is not None
    assert QtDatavisualization.QAbstract3DAxis is not None
    assert QtDatavisualization.Q3DScene is not None
    assert QtDatavisualization.QCategory3DAxis is not None
    assert QtDatavisualization.QAbstract3DSeries is not None
    assert QtDatavisualization.Q3DObject is not None
    assert QtDatavisualization.QCustom3DLabel is not None
    assert QtDatavisualization.Q3DSurface is not None
    assert QtDatavisualization.QLogValue3DAxisFormatter is not None



# -----------------------------------------------------------------------------
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""QtHelp Wrapper."""

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6, QtBindingsNotFoundError

if PYQT5:
    from PyQt5.QtHelp import *
elif PYQT6:
    from PyQt6.QtHelp import *
elif PYSIDE2:
    from PySide2.QtHelp import *
elif PYSIDE6:
    from PySide6.QtHelp import *
else:
    raise QtBindingsNotFoundError()



import os
import sys
import subprocess

import pytest

from qtpy import QtCore, QtGui, QtWidgets, API_NAMES, PythonQtValueError

try:
    # removed in qt 6.0
    from qtpy import QtWebEngineWidgets
except Exception:
    pass

def assert_pyside2():
    """
    Make sure that we are using PySide
    """
    import PySide2
    assert QtCore.QEvent is PySide2.QtCore.QEvent
    assert QtGui.QPainter is PySide2.QtGui.QPainter
    assert QtWidgets.QWidget is PySide2.QtWidgets.QWidget
    assert QtWebEngineWidgets.QWebEnginePage is PySide2.QtWebEngineWidgets.QWebEnginePage
    assert os.environ['QT_API'] == 'pyside2'

def assert_pyside6():
    """
    Make sure that we are using PySide
    """
    import PySide6
    assert QtCore.QEvent is PySide6.QtCore.QEvent
    assert QtGui.QPainter is PySide6.QtGui.QPainter
    assert QtWidgets.QWidget is PySide6.QtWidgets.QWidget
    # Only valid for qt>=6.2
    # assert QtWebEngineWidgets.QWebEnginePage is PySide6.QtWebEngineCore.QWebEnginePage
    assert os.environ['QT_API'] == 'pyside6'

def assert_pyqt5():
    """
    Make sure that we are using PyQt5
    """
    import PyQt5
    assert QtCore.QEvent is PyQt5.QtCore.QEvent
    assert QtGui.QPainter is PyQt5.QtGui.QPainter
    assert QtWidgets.QWidget is PyQt5.QtWidgets.QWidget
    assert os.environ['QT_API'] == 'pyqt5'

def assert_pyqt6():
    """
    Make sure that we are using PyQt6
    """
    import PyQt6
    assert QtCore.QEvent is PyQt6.QtCore.QEvent
    assert QtGui.QPainter is PyQt6.QtGui.QPainter
    assert QtWidgets.QWidget is PyQt6.QtWidgets.QWidget
    assert os.environ['QT_API'] == 'pyqt6'


def test_qt_api():
    """
    If QT_API is specified, we check that the correct Qt wrapper was used
    """

    QT_API = os.environ.get('QT_API', '').lower()

    if QT_API == 'pyqt5':
        assert_pyqt5()
    elif QT_API == 'pyside2':
        assert_pyside2()
    elif QT_API == 'pyqt6':
        assert_pyqt6()
    elif QT_API == 'pyside6':
        assert_pyside6()
    else:
        # If the tests are run locally, USE_QT_API and QT_API may not be
        # defined, but we still want to make sure qtpy is behaving sensibly.
        # We should then be loading, in order of decreasing preference, PyQt5,
        # PySide2, PyQt6 and PySide6.
        try:
            import PyQt5
        except ImportError:
            try:
                import PySide2
            except ImportError:
                try:
                    import PyQt6
                except ImportError:
                    import PySide6
                    assert_pyside6()
                else:
                    assert_pyqt6()
            else:
                assert_pyside2()
        else:
            assert_pyqt5()


@pytest.mark.parametrize('api', API_NAMES.values())
def test_qt_api_environ(api):
    """
    If no QT_API is specified but some Qt is imported, ensure QT_API is set properly.
    """
    mod = f'{api}.QtCore'
    pytest.importorskip(mod, reason=f'Requires {api}')
    # clean env
    env = os.environ.copy()
    for key in ('QT_API', 'USE_QT_API'):
        if key in env:
            del env[key]
    cmd = f"""
import {mod}
from qtpy import API
import os
print(API)
print(os.environ['QT_API'])
"""
    output = subprocess.check_output([sys.executable, '-c', cmd], env=env)
    got_api, env_qt_api = output.strip().decode('utf-8').splitlines()
    assert got_api == api.lower()
    assert env_qt_api == api.lower()
    # Also ensure we raise a nice error
    env['QT_API'] = 'bad'
    cmd = """
try:
    import qtpy
except ValueError as exc:
    assert 'Specified QT_API' in str(exc), str(exc)
else:
    raise AssertionError('QtPy imported despite bad QT_API')
"""
    subprocess.check_call([sys.executable, '-Oc', cmd], env=env)



import sys

import pytest

from qtpy import PYSIDE6, PYQT6


@pytest.mark.skipif(
    sys.platform.startswith("linux") and (PYSIDE6 or PYQT6),
    reason="Needs to setup GStreamer on Linux",
)
def test_qtmultimedia():
    """Test the qtpy.QtMultimedia namespace"""
    from qtpy import QtMultimedia

    assert QtMultimedia.QAudio is not None
    assert QtMultimedia.QAudioInput is not None

    if not (PYSIDE6 or PYQT6):
        assert QtMultimedia.QAbstractVideoBuffer is not None
        assert QtMultimedia.QAudioDeviceInfo is not None
        assert QtMultimedia.QSound is not None



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder Development Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides QtGui classes and functions."""

from . import PYQT6, PYQT5, PYSIDE2, PYSIDE6, QtBindingsNotFoundError

if PYQT5:
    from PyQt5.QtGui import *
elif PYQT6:
    from PyQt6 import QtGui
    from PyQt6.QtGui import *
    from PyQt6.QtOpenGL import *
    QFontMetrics.width = lambda self, *args, **kwargs: self.horizontalAdvance(*args, **kwargs)

    # Map missing/renamed methods
    QDrag.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QGuiApplication.exec_ = QGuiApplication.exec
    QTextDocument.print_ = lambda self, *args, **kwargs: self.print(*args, **kwargs)

    # Allow unscoped access for enums inside the QtGui module
    from .enums_compat import promote_enums
    promote_enums(QtGui)
    del QtGui
elif PYSIDE2:
    from PySide2.QtGui import *
    if hasattr(QFontMetrics, 'horizontalAdvance'):
        # Needed to prevent raising a DeprecationWarning when using QFontMetrics.width
        QFontMetrics.width = lambda self, *args, **kwargs: self.horizontalAdvance(*args, **kwargs)
elif PYSIDE6:
    from PySide6.QtGui import *
    from PySide6.QtOpenGL import *
    QFontMetrics.width = lambda self, *args, **kwargs: self.horizontalAdvance(*args, **kwargs)

    # Map DeprecationWarning methods
    QDrag.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QGuiApplication.exec_ = QGuiApplication.exec
else:
    raise QtBindingsNotFoundError()

if PYSIDE2 or PYSIDE6:
    # PySide{2,6} do not accept the `mode` keyword argument in
    # QTextCursor.movePosition() even though it is a valid optional argument
    # as per C++ API. Fix this by monkeypatching.
    #
    # Notes:
    #
    # * The `mode` argument is called `arg__2` in PySide{2,6} as per
    #   QTextCursor.movePosition.__doc__ and __signature__. Using `arg__2` as
    #   keyword argument works as intended, so does using a positional
    #   argument. Tested with PySide2 5.15.0, 5.15.2.1 and 5.15.3 and PySide6
    #   6.3.0; older version, down to PySide 1, are probably affected as well [1].
    #
    # * PySide2 5.15.0 and 5.15.2.1 silently ignore invalid keyword arguments,
    #   i.e. passing the `mode` keyword argument has no effect and doesn’t
    #   raise an exception. Older versions, down to PySide 1, are probably
    #   affected as well [1]. At least PySide2 5.15.3 and PySide6 6.3.0 raise an
    #   exception when `mode` or any other invalid keyword argument is passed.
    #
    # [1] https://bugreports.qt.io/browse/PYSIDE-185
    movePosition = QTextCursor.movePosition
    def movePositionPatched(
        self,
        operation: QTextCursor.MoveOperation,
        mode: QTextCursor.MoveMode = QTextCursor.MoveAnchor,
        n: int = 1,
    ) -> bool:
        return movePosition(self, operation, mode, n)
    QTextCursor.movePosition = movePositionPatched



# -----------------------------------------------------------------------------
# Copyright © 2014-2015 Colin Duquesnoy
# Copyright © 2009- The Spyder Developmet Team
#
# Licensed under the terms of the MIT License
# (see LICENSE.txt for details)
# -----------------------------------------------------------------------------

"""Provides widget classes and functions."""

from . import PYQT5, PYQT6, PYSIDE2, PYSIDE6, QtBindingsNotFoundError

if PYQT5:
    from PyQt5.QtWidgets import *
elif PYQT6:
    from PyQt6 import QtWidgets
    from PyQt6.QtWidgets import *
    from PyQt6.QtGui import QAction, QActionGroup, QShortcut, QFileSystemModel
    from PyQt6.QtOpenGLWidgets import QOpenGLWidget

    # Map missing/renamed methods
    QTextEdit.setTabStopWidth = lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    QTextEdit.tabStopWidth = lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    QTextEdit.print_ = lambda self, *args, **kwargs: self.print(*args, **kwargs)
    QPlainTextEdit.setTabStopWidth = lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    QPlainTextEdit.tabStopWidth = lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    QPlainTextEdit.print_ = lambda self, *args, **kwargs: self.print(*args, **kwargs)
    QApplication.exec_ = QApplication.exec
    QDialog.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QMenu.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)

    # Allow unscoped access for enums inside the QtWidgets module
    from .enums_compat import promote_enums
    promote_enums(QtWidgets)
    del QtWidgets
elif PYSIDE2:
    from PySide2.QtWidgets import *
elif PYSIDE6:
    from PySide6.QtWidgets import *
    from PySide6.QtGui import QAction, QActionGroup, QShortcut
    from PySide6.QtOpenGLWidgets import QOpenGLWidget

    # Map missing/renamed methods
    QTextEdit.setTabStopWidth = lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    QTextEdit.tabStopWidth = lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)
    QPlainTextEdit.setTabStopWidth = lambda self, *args, **kwargs: self.setTabStopDistance(*args, **kwargs)
    QPlainTextEdit.tabStopWidth = lambda self, *args, **kwargs: self.tabStopDistance(*args, **kwargs)

    # Map DeprecationWarning methods
    QApplication.exec_ = QApplication.exec
    QDialog.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
    QMenu.exec_ = lambda self, *args, **kwargs: self.exec(*args, **kwargs)
else:
    raise QtBindingsNotFoundError()



import pytest
from packaging import version
from qtpy import PYQT5, PYSIDE2, PYQT_VERSION


@pytest.mark.skipif(
    not ((PYQT5 and version.parse(PYQT_VERSION) >= version.parse("5.15.1")) or PYSIDE2),
    reason="Only available in Qt5 bindings (PyQt5 >= 5.15.1 or PySide2)",
)
def test_qttexttospeech():
    """Test the qtpy.QtTextToSpeech namespace."""
    from qtpy import QtTextToSpeech

    assert QtTextToSpeech.QTextToSpeech is not None
    assert QtTextToSpeech.QVoice is not None

    if PYSIDE2:
        assert QtTextToSpeech.QTextToSpeechEngine is not None



"""Test QtGui."""

import sys

import pytest

from qtpy import PYQT5, PYQT_VERSION, PYSIDE2, PYSIDE6, QtGui
from qtpy.tests.utils import not_using_conda


@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Fatal Python error: Aborted on Linux CI when not using conda")
def test_qfontmetrics_width(qtbot):
    """Test QFontMetrics width"""
    assert QtGui.QFontMetrics.width is not None
    font = QtGui.QFont("times", 24)
    font_metrics = QtGui.QFontMetrics(font)
    width = font_metrics.width("Test")
    assert width in range(40, 62)


@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Fatal Python error: Aborted on Linux CI when not using conda")
def test_qdrag_functions(qtbot):
    """Test functions mapping for QtGui.QDrag."""
    assert QtGui.QDrag.exec_ is not None
    drag = QtGui.QDrag(None)
    drag.exec_()


def test_qguiapplication_functions():
    """Test functions mapping for QtGui.QGuiApplication."""
    assert QtGui.QGuiApplication.exec_ is not None


@pytest.mark.skipif(
    sys.platform.startswith('linux') and not_using_conda(),
    reason="Segmentation fault/Aborted on Linux CI when not using conda")
def test_qtextdocument_functions(pdf_writer):
    """Test functions mapping for QtGui.QTextDocument."""
    assert QtGui.QTextDocument.print_ is not None
    text_document = QtGui.QTextDocument("Test")
    print_device, output_path = pdf_writer
    text_document.print_(print_device)
    assert output_path.exists()


@pytest.mark.skipif(PYQT5 and PYQT_VERSION.startswith('5.9'),
                    reason="A specific setup with at least sip 4.9.9 is needed for PyQt5 5.9.*"
                           "to work with scoped enum access")
def test_enum_access():
    """Test scoped and unscoped enum access for qtpy.QtWidgets.*."""
    assert QtGui.QColor.Rgb == QtGui.QColor.Spec.Rgb
    assert QtGui.QFont.AllUppercase == QtGui.QFont.Capitalization.AllUppercase
    assert QtGui.QIcon.Normal == QtGui.QIcon.Mode.Normal
    assert QtGui.QImage.Format_Invalid == QtGui.QImage.Format.Format_Invalid

@pytest.mark.skipif(not (PYSIDE2 or PYSIDE6), reason="PySide{2,6} specific test")
def test_qtextcursor_moveposition():
    """Test monkeypatched QTextCursor.movePosition"""
    doc = QtGui.QTextDocument("foo bar baz")
    cursor = QtGui.QTextCursor(doc)

    assert not cursor.movePosition(QtGui.QTextCursor.Start)
    assert cursor.movePosition(QtGui.QTextCursor.EndOfWord, mode=QtGui.QTextCursor.KeepAnchor)
    assert cursor.selectedText() == "foo"

    assert cursor.movePosition(QtGui.QTextCursor.Start)
    assert cursor.movePosition(QtGui.QTextCursor.WordRight, n=2, mode=QtGui.QTextCursor.KeepAnchor)
    assert cursor.selectedText() == "foo bar "

    assert cursor.movePosition(QtGui.QTextCursor.Start)
    assert cursor.position() == cursor.anchor()
    assert cursor.movePosition(QtGui.QTextCursor.NextWord, QtGui.QTextCursor.KeepAnchor, 3)
    assert cursor.selectedText() == "foo bar baz"



"""Test QtMultimediaWidgets."""

import pytest

from qtpy import PYQT5, PYSIDE2
from qtpy.tests.utils import using_conda


@pytest.mark.skipif(not (PYQT5 or PYSIDE2), reason="Only available in Qt5 bindings")
@pytest.mark.skipif(
    using_conda(), reason="Conda packages don't seem to include QtMultimedia"
)
def test_qtmultimediawidgets():
    """Test the qtpy.QtMultimediaWidgets namespace"""
    from qtpy import QtMultimediaWidgets

    assert QtMultimediaWidgets.QCameraViewfinder is not None
    assert QtMultimediaWidgets.QGraphicsVideoItem is not None
    assert QtMultimediaWidgets.QVideoWidget is not None
    # assert QtMultimediaWidgets.QVideoWidgetControl is not None
